"""erasus.strategies.gradient_methods"""

from erasus.strategies.gradient_methods.gradient_ascent import GradientAscentStrategy
from erasus.strategies.gradient_methods.modality_decoupling import ModalityDecouplingStrategy
from erasus.strategies.gradient_methods.weighted_gradient_ascent import (
    FPGAStrategy,
    WGAStrategy,
)

__all__ = [
    "GradientAscentStrategy",
    "ModalityDecouplingStrategy",
    "WGAStrategy",
    "FPGAStrategy",
]
