"""erasus.strategies.vlm_specific — VLM-specific unlearning strategies."""

from erasus.strategies.gradient_methods.modality_decoupling import ModalityDecouplingStrategy

__all__ = ["ModalityDecouplingStrategy"]
