"""
erasus.models.video — Video model wrappers.
"""

from erasus.models.video.videomae import VideoMAEWrapper
from erasus.models.video.video_clip import VideoCLIPWrapper

__all__ = ["VideoMAEWrapper", "VideoCLIPWrapper"]
