"""Cross-modal decoupling — re-exports ModalityDecouplingStrategy."""

from erasus.strategies.gradient_methods.modality_decoupling import ModalityDecouplingStrategy

__all__ = ["ModalityDecouplingStrategy"]
